@@ -1,15 +1,3 @@
-use std::cmp;
-
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
-use crate::CliResult;
-use csv::ByteRecord;
-use rayon::prelude::*;
-use serde::Deserialize;
-
-use crate::cmd::sort::iter_cmp;
-
 static USAGE: &str = r#"
 Deduplicates CSV rows. 
 
@@ -24,15 +12,18 @@ Either way, the output will not only be deduplicated, it will also be sorted.
 
 A duplicate count will also be sent to <stderr>.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_dedup.rs.
+
 Usage:
     qsv dedup [options] [<input>]
+    qsv dedup --help
 
 sort options:
     -s, --select <arg>         Select a subset of columns to dedup.
                                Note that the outputs will remain at the full width
                                of the CSV.
                                See 'qsv select --help' for the format details.
-    -C, --no-case              Compare strings disregarding case 
+    -i, --ignore-case          Compare strings disregarding case.
     --sorted                   The input is already sorted. Do not load the CSV into
                                memory to sort it first. Meant to be used in tandem and
                                after an extsort.
@@ -56,23 +47,35 @@ Common options:
                                Must be a single character. (default: ,)
 "#;
 
+use std::cmp;
+
+use csv::ByteRecord;
+use rayon::prelude::*;
+use serde::Deserialize;
+
+use crate::{
+    cmd::sort::iter_cmp,
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util, CliResult,
+};
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
-    flag_select: SelectColumns,
-    flag_no_case: bool,
-    flag_sorted: bool,
-    flag_dupes_output: Option<String>,
-    flag_output: Option<String>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+    arg_input:           Option<String>,
+    flag_select:         SelectColumns,
+    flag_ignore_case:    bool,
+    flag_sorted:         bool,
+    flag_dupes_output:   Option<String>,
+    flag_output:         Option<String>,
+    flag_no_headers:     bool,
+    flag_delimiter:      Option<Delimiter>,
     flag_human_readable: bool,
-    flag_jobs: Option<usize>,
+    flag_jobs:           Option<usize>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
-    let no_case = args.flag_no_case;
+    let ignore_case = args.flag_ignore_case;
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
@@ -105,8 +108,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             };
             let a = sel.select(&record);
             let b = sel.select(&next_record);
-            let comparison = if no_case {
-                iter_cmp_no_case(a, b)
+            let comparison = if ignore_case {
+                iter_cmp_ignore_case(a, b)
             } else {
                 iter_cmp(a, b)
             };
@@ -122,10 +125,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     record.clone_from(&next_record);
                 }
                 cmp::Ordering::Greater => {
-                    let fail_msg = format!(
+                    return fail_clierror!(
                         "Aborting! Input not sorted! {record:?} is greater than {next_record:?}"
                     );
-                    return fail!(fail_msg);
                 }
             }
         }
@@ -143,8 +145,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         while current + 1 < all.len() {
             let a = sel.select(&all[current]);
             let b = sel.select(&all[current + 1]);
-            if no_case {
-                if iter_cmp_no_case(a, b) == cmp::Ordering::Equal {
+            if ignore_case {
+                if iter_cmp_ignore_case(a, b) == cmp::Ordering::Equal {
                     dupe_count += 1;
                     if dupes_output {
                         dupewtr.write_byte_record(&all[current])?;
@@ -180,7 +182,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
 /// Try comparing `a` and `b` ignoring the case
 #[inline]
-pub fn iter_cmp_no_case<'a, L, R>(mut a: L, mut b: R) -> cmp::Ordering
+pub fn iter_cmp_ignore_case<'a, L, R>(mut a: L, mut b: R) -> cmp::Ordering
 where
     L: Iterator<Item = &'a [u8]>,
     R: Iterator<Item = &'a [u8]>,
