@@ -1,11 +1,3 @@
-use uuid::Uuid;
-
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
 static USAGE: &str = r#"
 Add a new column enumerating the lines of a CSV file. This can be useful to keep
 track of a specific line order, give a unique identifier to each line or even
@@ -42,7 +34,7 @@ enum options:
     --copy <column>          Name of a column to copy.
                              Changes the default column name to "{column}_copy".
     --uuid                   When set, the column will be populated with
-                             uuids (v4) instead of the incremental identifer.
+                             uuids (v4) instead of the incremental identifier.
                              Changes the default column name to "uuid".
 
 Common options:
@@ -54,24 +46,34 @@ Common options:
                              Must be a single character. (default: ,)
 "#;
 
+use serde::Deserialize;
+use uuid::Uuid;
+
+use crate::{
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util, CliResult,
+};
+
 const NULL_VALUE: &str = "<NULL>";
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
+    arg_input:       Option<String>,
     flag_new_column: Option<String>,
-    flag_constant: Option<String>,
-    flag_copy: Option<SelectColumns>,
-    flag_uuid: bool,
-    flag_output: Option<String>,
+    flag_constant:   Option<String>,
+    flag_copy:       Option<SelectColumns>,
+    flag_uuid:       bool,
+    flag_output:     Option<String>,
     flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+    flag_delimiter:  Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
     let mut rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
+        .checkutf8(false)
         .no_headers(args.flag_no_headers);
 
     let mut rdr = rconfig.reader()?;
@@ -97,8 +99,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         } else if args.flag_constant.is_some() {
             headers.push_field(b"constant");
         } else if copy_operation {
-            let current_header = String::from_utf8(headers[copy_index].to_vec())
-                .expect("Could not parse cell as utf-8!");
+            let current_header = match String::from_utf8(headers[copy_index].to_vec()) {
+                Ok(s) => s,
+                Err(e) => return fail_clierror!("Could not parse cell as utf-8!: {e}"),
+            };
             headers.push_field(format!("{current_header}_copy").as_bytes());
         } else {
             headers.push_field(b"index");
@@ -118,6 +122,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 record.push_field(constant_value.as_bytes());
             }
         } else if copy_operation {
+            #[allow(clippy::unnecessary_to_owned)]
             record.push_field(&record[copy_index].to_vec());
         } else if args.flag_uuid {
             let id = Uuid::new_v4();
