@@ -1,26 +1,10 @@
-use ahash::AHashMap;
-use std::collections::hash_map::Entry;
-use std::fmt;
-use std::fs;
-use std::io;
-use std::str;
-
-use byteorder::{BigEndian, WriteBytesExt};
-
-use crate::config::{Config, Delimiter};
-use crate::index::Indexed;
-use crate::select::{SelectColumns, Selection};
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Removes a set of CSV data from another set based on the specified columns.
 
 Also can compute the intersection of two CSV sets with the -v flag.
 
 Matching is always done by ignoring leading and trailing whitespace. By default,
-matching is done case sensitively, but this can be disabled with the --no-case
+matching is done case sensitively, but this can be disabled with the --ignore-case
 flag.
 
 The columns arguments specify the columns to match for each input. Columns can
@@ -29,12 +13,14 @@ separating them with a comma. Specify a range of columns with `-`. Both
 columns1 and columns2 must specify exactly the same number of columns.
 (See 'qsv select --help' for the full syntax.)
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_exclude.rs.
+
 Usage:
     qsv exclude [options] <columns1> <input1> <columns2> <input2>
     qsv exclude --help
 
 exclude options:
-    --no-case              When set, matching is done case insensitively.
+    -i, --ignore-case      When set, matching is done case insensitively.
     -v                     When set, matching rows will be the only ones included,
                            forming set intersection, instead of the ones discarded.
 
@@ -46,21 +32,34 @@ Common options:
                            sliced, etc.)
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::{collections::hash_map::Entry, fmt, fs, io, str};
+
+use ahash::AHashMap;
+use byteorder::{BigEndian, WriteBytesExt};
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    index::Indexed,
+    select::{SelectColumns, Selection},
+    util, CliResult,
+};
 
 type ByteString = Vec<u8>;
 
 #[derive(Deserialize)]
 struct Args {
-    arg_columns1: SelectColumns,
-    arg_input1: String,
-    arg_columns2: SelectColumns,
-    arg_input2: String,
-    flag_v: bool,
-    flag_output: Option<String>,
-    flag_no_headers: bool,
-    flag_no_case: bool,
-    flag_delimiter: Option<Delimiter>,
+    arg_columns1:     SelectColumns,
+    arg_input1:       String,
+    arg_columns2:     SelectColumns,
+    arg_input2:       String,
+    flag_v:           bool,
+    flag_output:      Option<String>,
+    flag_no_headers:  bool,
+    flag_ignore_case: bool,
+    flag_delimiter:   Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -71,13 +70,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 }
 
 struct IoState<R, W: io::Write> {
-    wtr: csv::Writer<W>,
-    rdr1: csv::Reader<R>,
-    sel1: Selection,
-    rdr2: csv::Reader<R>,
-    sel2: Selection,
+    wtr:        csv::Writer<W>,
+    rdr1:       csv::Reader<R>,
+    sel1:       Selection,
+    rdr2:       csv::Reader<R>,
+    sel2:       Selection,
     no_headers: bool,
-    casei: bool,
+    casei:      bool,
 }
 
 impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
@@ -90,25 +89,20 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
     }
 
     fn exclude(mut self, invert: bool) -> CliResult<()> {
-        let _scratch = csv::ByteRecord::new();
+        // amortize allocations
+        #[allow(unused_assignments)]
+        let mut curr_row = csv::ByteRecord::new();
+
         let validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei)?;
         for row in self.rdr1.byte_records() {
-            let row = row?;
-            let key = get_row_key(&self.sel1, &row, self.casei);
-            match validx.values.get(&key) {
-                None => {
-                    if invert {
-                        continue;
-                    }
-                    self.wtr.write_record(row.iter())?;
-                }
-                Some(_rows) => {
-                    if invert {
-                        self.wtr.write_record(row.iter())?;
-                    } else {
-                        continue;
-                    }
+            curr_row = row?;
+            let key = get_row_key(&self.sel1, &curr_row, self.casei);
+            if let Some(_rows) = validx.values.get(&key) {
+                if invert {
+                    self.wtr.write_record(curr_row.iter())?;
                 }
+            } else if !invert {
+                self.wtr.write_record(curr_row.iter())?;
             }
         }
         Ok(())
@@ -136,10 +130,11 @@ impl Args {
             rdr2,
             sel2,
             no_headers: rconf1.no_headers,
-            casei: self.flag_no_case,
+            casei: self.flag_ignore_case,
         })
     }
 
+    #[allow(clippy::unused_self)]
     fn get_selections<R: io::Read>(
         &self,
         rconf1: &Config,
@@ -152,12 +147,12 @@ impl Args {
         let select1 = rconf1.selection(headers1)?;
         let select2 = rconf2.selection(headers2)?;
         if select1.len() != select2.len() {
-            return fail!(format!(
-                "Column selections must have the same number of columns, \
-                 but found column selections with {} and {} columns.",
+            return fail_clierror!(
+                "Column selections must have the same number of columns, but found column \
+                 selections with {} and {} columns.",
                 select1.len(),
                 select2.len()
-            ));
+            );
         }
         Ok((select1, select2))
     }
@@ -166,8 +161,8 @@ impl Args {
 #[allow(dead_code)]
 struct ValueIndex<R> {
     // This maps tuples of values to corresponding rows.
-    values: AHashMap<Vec<ByteString>, Vec<usize>>,
-    idx: Indexed<R, io::Cursor<Vec<u8>>>,
+    values:   AHashMap<Vec<ByteString>, Vec<usize>>,
+    idx:      Indexed<R, io::Cursor<Vec<u8>>>,
     num_rows: usize,
 }
 
@@ -239,7 +234,7 @@ impl<R> fmt::Debug for ValueIndex<R> {
                 .iter()
                 .map(|k| String::from_utf8(k.clone()).unwrap())
                 .collect::<Vec<_>>();
-            writeln!(f, "({}) => {:?}", keys.join(", "), rows)?;
+            writeln!(f, "({}) => {rows:?}", keys.join(", "))?;
         }
         Ok(())
     }
