@@ -1,14 +1,4 @@
-use std::fs;
-use std::io::{self, prelude::*};
-use std::path;
-
-use crate::util;
-use crate::CliResult;
-use ext_sort::{buffer::mem::MemoryLimitedBufferBuilder, ExternalSorter, ExternalSorterBuilder};
-use serde::Deserialize;
-use sysinfo::{System, SystemExt};
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Sort an arbitrarily large CSV/text file using a multithreaded external sort algorithm.
 
 This command does not work with <stdin>/<stdout>. Valid input, and output
@@ -20,6 +10,7 @@ otherwise, the first line will not be included in the external sort.
 
 Usage:
     qsv extsort [options] <input> <output>
+    qsv extsort --help
 
 External sort option:
     -j, --jobs <arg>       The number of jobs to run in parallel.
@@ -32,13 +23,25 @@ Common options:
                            as headers and will be sorted with the rest
                            of the rows. Otherwise, the first row will always
                            appear as the header row in the output.
-";
+"#;
+
+use std::{
+    fs,
+    io::{self, prelude::*},
+    path,
+};
+
+use ext_sort::{buffer::mem::MemoryLimitedBufferBuilder, ExternalSorter, ExternalSorterBuilder};
+use serde::Deserialize;
+use sysinfo::{System, SystemExt};
+
+use crate::{util, CliResult};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: String,
-    arg_output: String,
-    flag_jobs: Option<usize>,
+    arg_input:       String,
+    arg_output:      String,
+    flag_jobs:       Option<usize>,
     flag_no_headers: bool,
 }
 
@@ -60,25 +63,39 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     };
     log::info!("{mem_limited_buffer} bytes used for in memory mergesort buffer...");
 
-    let mut input_reader = io::BufReader::new(fs::File::open(&args.arg_input)?);
+    let mut input_reader = io::BufReader::new(match fs::File::open(&args.arg_input) {
+        Ok(f) => f,
+        Err(e) => return fail_clierror!("Cannot read input file {e}"),
+    });
+
+    let mut output_writer = io::BufWriter::new(match fs::File::create(&args.arg_output) {
+        Ok(f) => f,
+        Err(e) => return fail_clierror!("Cannot create output file: {e}"),
+    });
 
     let sorter: ExternalSorter<String, io::Error, MemoryLimitedBufferBuilder> =
-        ExternalSorterBuilder::new()
+        match ExternalSorterBuilder::new()
             .with_tmp_dir(path::Path::new("./"))
             .with_buffer(MemoryLimitedBufferBuilder::new(mem_limited_buffer))
             .with_rw_buf_size(RW_BUFFER_CAPACITY)
             .with_threads_number(util::njobs(args.flag_jobs))
             .build()
-            .unwrap();
+        {
+            Ok(sorter) => sorter,
+            Err(e) => {
+                return fail_clierror!("cannot create external sorter: {e}");
+            }
+        };
 
     let mut header = String::new();
     if !args.flag_no_headers {
         input_reader.read_line(&mut header)?;
     }
 
-    let sorted = sorter.sort(input_reader.lines()).unwrap();
+    let Ok(sorted) = sorter.sort(input_reader.lines()) else {
+                 return fail!("cannot do external sort");
+             };
 
-    let mut output_writer = io::BufWriter::new(fs::File::create(&args.arg_output)?);
     if !header.is_empty() {
         output_writer.write_all(format!("{}\n", header.trim_end()).as_bytes())?;
     }
@@ -89,3 +106,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     output_writer.flush()?;
     Ok(())
 }
+
+#[test]
+fn test_mem_check() {
+    // check to see if sysinfo return meminfo without segfaulting
+    let mut sys = System::new_all();
+    sys.refresh_memory();
+    let mem10percent = (sys.total_memory() * 1000) / 10; // 10 percent of total memory
+    assert!(mem10percent > 0);
+}
