@@ -1,14 +1,4 @@
-use std::borrow::Cow;
-use std::io::{self, Write};
-
-use tabwriter::TabWriter;
-
-use crate::config::{Config, Delimiter};
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Prints flattened records such that fields are labeled separated by a new line.
 This mode is particularly useful for viewing one record at a time. Each
 record is separated by a special '#' character (on a line by itself), which
@@ -17,8 +7,11 @@ can be changed with the --separator flag.
 There is also a condensed view (-c or --condense) that will shorten the
 contents of each field to provide a summary view.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_flatten.rs.
+
 Usage:
     qsv flatten [options] [<input>]
+    qsv flatten --help
 
 flatten options:
     -c, --condense <arg>  Limits the length of each field to the value
@@ -37,21 +30,35 @@ Common options:
                            will be its index.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::{
+    borrow::Cow,
+    io::{self, Write},
+};
+
+use serde::Deserialize;
+use tabwriter::TabWriter;
+
+use crate::{
+    config::{Config, Delimiter},
+    util, CliResult,
+};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
-    flag_condense: Option<usize>,
-    flag_separator: String,
+    arg_input:       Option<String>,
+    flag_condense:   Option<usize>,
+    flag_separator:  String,
     flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+    flag_delimiter:  Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
+        .checkutf8(false)
         .no_headers(args.flag_no_headers);
     let mut rdr = rconfig.reader()?;
     let headers = rdr.byte_headers()?.clone();
@@ -66,12 +73,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let r = r?;
         for (i, (header, field)) in headers.iter().zip(&r).enumerate() {
             if rconfig.no_headers {
-                write!(&mut wtr, "{}", i)?;
+                write!(&mut wtr, "{i}")?;
             } else {
                 wtr.write_all(header)?;
             }
             wtr.write_all(b"\t")?;
-            wtr.write_all(&*util::condense(Cow::Borrowed(field), args.flag_condense))?;
+            wtr.write_all(&util::condense(Cow::Borrowed(field), args.flag_condense))?;
             wtr.write_all(b"\n")?;
         }
     }
