@@ -1,17 +1,4 @@
-use std::fs;
-use std::io;
-
-use stats::{merge_all, Frequencies};
-use threadpool::ThreadPool;
-
-use crate::config::{Config, Delimiter};
-use crate::index::Indexed;
-use crate::select::{SelectColumns, Selection};
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Compute a frequency table on CSV data.
 
 The frequency table is formatted as CSV data:
@@ -25,8 +12,11 @@ respectively.
 Since this computes an exact frequency table, memory proportional to the
 cardinality of each column is required.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_frequency.rs.
+
 Usage:
     qsv frequency [options] [<input>]
+    qsv frequency --help
 
 frequency options:
     -s, --select <arg>     Select a subset of columns to compute frequencies
@@ -44,11 +34,8 @@ frequency options:
                            This works better when the given CSV data has
                            an index already created. Note that a file handle
                            is opened for each job.
-                           When set to '0', the number of jobs is set to the
-                           number of CPUs detected divided by 3.
-                           When set to '-1', the number of jobs is set to the
+                           When not set, the number of jobs is set to the
                            number of CPUs detected.
-                           [default: 0]
 
 Common options:
     -h, --help             Display this message
@@ -59,19 +46,32 @@ Common options:
                            names.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::{fs, io};
+
+use serde::Deserialize;
+use stats::{merge_all, Frequencies};
+use threadpool::ThreadPool;
+
+use crate::{
+    config::{Config, Delimiter},
+    index::Indexed,
+    select::{SelectColumns, Selection},
+    util, CliResult,
+};
 
 #[derive(Clone, Deserialize)]
 pub struct Args {
-    pub arg_input: Option<String>,
-    pub flag_select: SelectColumns,
-    pub flag_limit: usize,
-    pub flag_asc: bool,
-    pub flag_no_nulls: bool,
-    pub flag_jobs: isize,
-    pub flag_output: Option<String>,
+    pub arg_input:       Option<String>,
+    pub flag_select:     SelectColumns,
+    pub flag_limit:      usize,
+    pub flag_asc:        bool,
+    pub flag_no_nulls:   bool,
+    pub flag_jobs:       Option<usize>,
+    pub flag_output:     Option<String>,
     pub flag_no_headers: bool,
-    pub flag_delimiter: Option<Delimiter>,
+    pub flag_delimiter:  Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -80,18 +80,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let mut wtr = Config::new(&args.flag_output).writer()?;
     let (headers, tables) = match args.rconfig().indexed()? {
-        Some(ref mut idx) if args.njobs() > 1 => args.parallel_ftables(idx),
+        Some(ref mut idx) if util::njobs(args.flag_jobs) > 1 => args.parallel_ftables(idx),
         _ => args.sequential_ftables(),
     }?;
 
     wtr.write_record(vec!["field", "value", "count"])?;
     let head_ftables = headers.into_iter().zip(tables.into_iter());
     for (i, (header, ftab)) in head_ftables.enumerate() {
-        let mut header = header.to_vec();
-        if rconfig.no_headers {
-            header = (i + 1).to_string().into_bytes();
-        }
-        for (value, count) in args.counts(&ftab).into_iter() {
+        let header = if rconfig.no_headers {
+            (i + 1).to_string().into_bytes()
+        } else {
+            header.to_vec()
+        };
+        for (value, count) in args.counts(&ftab) {
             let count = count.to_string();
             let row = vec![&*header, &*value, count.as_bytes()];
             wtr.write_record(row)?;
@@ -110,6 +111,7 @@ impl Args {
         Config::new(&self.arg_input)
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
+            .checkutf8(false)
             .select(self.flag_select.clone())
     }
 
@@ -151,10 +153,10 @@ impl Args {
             return Ok((headers, vec![]));
         }
 
-        let chunk_size = util::chunk_size(idx.count() as usize, self.njobs());
+        let chunk_size = util::chunk_size(idx.count() as usize, util::njobs(self.flag_jobs));
         let nchunks = util::num_of_chunks(idx.count() as usize, chunk_size);
 
-        let pool = ThreadPool::new(self.njobs());
+        let pool = ThreadPool::new(util::njobs(self.flag_jobs));
         let (send, recv) = channel::bounded(0);
         for i in 0..nchunks {
             let (send, args, sel) = (send.clone(), self.clone(), sel.clone());
@@ -196,20 +198,11 @@ impl Args {
     ) -> CliResult<(csv::ByteRecord, Selection)> {
         let headers = rdr.byte_headers()?;
         let sel = self.rconfig().selection(headers)?;
-        Ok((sel.select(headers).map(|h| h.to_vec()).collect(), sel))
-    }
-
-    fn njobs(&self) -> usize {
-        let num_cpus = util::num_cpus();
-        match self.flag_jobs {
-            0 => util::max_jobs(),
-            flag_jobs if flag_jobs < 0 => num_cpus,
-            flag_jobs if flag_jobs > num_cpus as isize => num_cpus,
-            _ => self.flag_jobs as usize,
-        }
+        Ok((sel.select(headers).map(<[u8]>::to_vec).collect(), sel))
     }
 }
 
+#[inline]
 fn trim(bs: ByteString) -> ByteString {
     match String::from_utf8(bs) {
         Ok(s) => s.trim().as_bytes().to_vec(),
