@@ -1,13 +1,4 @@
-use std::io;
-
-use tabwriter::TabWriter;
-
-use crate::config::Delimiter;
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Prints the fields of the first row in the CSV data.
 
 These names can be used in commands like 'select' to refer to columns in the
@@ -16,8 +7,11 @@ CSV data.
 Note that multiple CSV files may be given to this command. This is useful with
 the --intersect flag.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_headers.rs.
+
 Usage:
     qsv headers [options] [<input>...]
+    qsv headers --help
 
 headers options:
     -j, --just-names       Only show the header names (hide column index).
@@ -30,19 +24,26 @@ Common options:
     -h, --help             Display this message
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::io;
+
+use serde::Deserialize;
+use tabwriter::TabWriter;
+
+use crate::{config::Delimiter, util, CliResult};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Vec<String>,
+    arg_input:       Vec<String>,
     flag_just_names: bool,
-    flag_intersect: bool,
-    flag_delimiter: Option<Delimiter>,
+    flag_intersect:  bool,
+    flag_delimiter:  Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
-    let configs = util::many_configs(&*args.arg_input, args.flag_delimiter, true)?;
+    let configs = util::many_configs(&args.arg_input, args.flag_delimiter, true)?;
 
     let num_inputs = configs.len();
     let mut headers: Vec<Vec<u8>> = vec![];
