@@ -1,27 +1,11 @@
-use std::collections::hash_map::Entry;
-use std::fmt;
-use std::io;
-use std::iter::repeat;
-use std::str;
-
-use ahash::AHashMap;
-use byteorder::{BigEndian, WriteBytesExt};
-
-use crate::config::{Config, Delimiter, SeekRead};
-use crate::index::Indexed;
-use crate::select::{SelectColumns, Selection};
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Joins two sets of CSV data on the specified columns.
 
 The default join operation is an 'inner' join. This corresponds to the
 intersection of rows on the keys specified.
 
 Joins are always done by ignoring leading and trailing whitespace. By default,
-joins are done case sensitively, but this can be disabled with the --no-case
+joins are done case sensitively, but this can be disabled with the --ignore-case
 flag.
 
 The columns arguments specify the columns to join for each input. Columns can
@@ -30,6 +14,8 @@ separating them with a comma. Specify a range of columns with `-`. Both
 columns1 and columns2 must specify exactly the same number of columns.
 (See 'qsv select --help' for the full syntax.)
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_join.rs.
+
 Usage:
     qsv join [options] <columns1> <input1> <columns2> <input2>
     qsv join --help
@@ -39,7 +25,7 @@ input parameters:
     e.g. 'qsv frequency -s Agency nyc311.csv | qsv join value - id nycagencyinfo.csv'
 
 join options:
-    --no-case              When set, joins are done case insensitively.
+    -i, --ignore-case      When set, joins are done case insensitively.
     --left                 Do a 'left outer' join. This returns all rows in
                            first CSV data set, including rows with no
                            corresponding row in the second data set. When no
@@ -79,27 +65,38 @@ Common options:
                            sliced, etc.)
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
 
-type ByteString = Vec<u8>;
+use std::{collections::hash_map::Entry, fmt, io, iter::repeat, str};
+
+use ahash::AHashMap;
+use byteorder::{BigEndian, WriteBytesExt};
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter, SeekRead},
+    index::Indexed,
+    select::{SelectColumns, Selection},
+    util, CliResult,
+};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_columns1: SelectColumns,
-    arg_input1: String,
-    arg_columns2: SelectColumns,
-    arg_input2: String,
-    flag_left: bool,
-    flag_left_anti: bool,
-    flag_left_semi: bool,
-    flag_right: bool,
-    flag_full: bool,
-    flag_cross: bool,
-    flag_output: Option<String>,
-    flag_no_headers: bool,
-    flag_no_case: bool,
-    flag_nulls: bool,
-    flag_delimiter: Option<Delimiter>,
+    arg_columns1:     SelectColumns,
+    arg_input1:       String,
+    arg_columns2:     SelectColumns,
+    arg_input2:       String,
+    flag_left:        bool,
+    flag_left_anti:   bool,
+    flag_left_semi:   bool,
+    flag_right:       bool,
+    flag_full:        bool,
+    flag_cross:       bool,
+    flag_output:      Option<String>,
+    flag_no_headers:  bool,
+    flag_ignore_case: bool,
+    flag_nulls:       bool,
+    flag_delimiter:   Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -146,14 +143,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 }
 
 struct IoState<R, W: io::Write> {
-    wtr: csv::Writer<W>,
-    rdr1: csv::Reader<R>,
-    sel1: Selection,
-    rdr2: csv::Reader<R>,
-    sel2: Selection,
+    wtr:        csv::Writer<W>,
+    rdr1:       csv::Reader<R>,
+    sel1:       Selection,
+    rdr2:       csv::Reader<R>,
+    sel2:       Selection,
     no_headers: bool,
-    casei: bool,
-    nulls: bool,
+    casei:      bool,
+    nulls:      bool,
 }
 
 impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
@@ -180,16 +177,13 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
         for row in self.rdr1.byte_records() {
             let row = row?;
             let key = get_row_key(&self.sel1, &row, self.casei);
-            match validx.values.get(&key) {
-                None => continue,
-                Some(rows) => {
-                    for &rowi in rows.iter() {
-                        validx.idx.seek(rowi as u64)?;
+            if let Some(rows) = validx.values.get(&key) {
+                for &rowi in rows.iter() {
+                    validx.idx.seek(rowi as u64)?;
 
-                        validx.idx.read_byte_record(&mut scratch)?;
-                        let combined = row.iter().chain(scratch.iter());
-                        self.wtr.write_record(combined)?;
-                    }
+                    validx.idx.read_byte_record(&mut scratch)?;
+                    let combined = row.iter().chain(scratch.iter());
+                    self.wtr.write_record(combined)?;
                 }
             }
         }
@@ -208,26 +202,21 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
         for row in self.rdr1.byte_records() {
             let row = row?;
             let key = get_row_key(&self.sel1, &row, self.casei);
-            match validx.values.get(&key) {
-                None => {
+            if let Some(rows) = validx.values.get(&key) {
+                for &rowi in rows.iter() {
+                    validx.idx.seek(rowi as u64)?;
+                    let row1 = row.iter();
+                    validx.idx.read_byte_record(&mut scratch)?;
                     if right {
-                        self.wtr.write_record(pad2.iter().chain(&row))?;
+                        self.wtr.write_record(scratch.iter().chain(row1))?;
                     } else {
-                        self.wtr.write_record(row.iter().chain(&pad2))?;
-                    }
-                }
-                Some(rows) => {
-                    for &rowi in rows.iter() {
-                        validx.idx.seek(rowi as u64)?;
-                        let row1 = row.iter();
-                        validx.idx.read_byte_record(&mut scratch)?;
-                        if right {
-                            self.wtr.write_record(scratch.iter().chain(row1))?;
-                        } else {
-                            self.wtr.write_record(row1.chain(&scratch))?;
-                        }
+                        self.wtr.write_record(row1.chain(&scratch))?;
                     }
                 }
+            } else if right {
+                self.wtr.write_record(pad2.iter().chain(&row))?;
+            } else {
+                self.wtr.write_record(row.iter().chain(&pad2))?;
             }
         }
         Ok(())
@@ -267,19 +256,16 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
         for row1 in self.rdr1.byte_records() {
             let row1 = row1?;
             let key = get_row_key(&self.sel1, &row1, self.casei);
-            match validx.values.get(&key) {
-                None => {
-                    self.wtr.write_record(row1.iter().chain(&pad2))?;
-                }
-                Some(rows) => {
-                    for &rowi in rows.iter() {
-                        rdr2_written[rowi] = true;
+            if let Some(rows) = validx.values.get(&key) {
+                for &rowi in rows.iter() {
+                    rdr2_written[rowi] = true;
 
-                        validx.idx.seek(rowi as u64)?;
-                        validx.idx.read_byte_record(&mut scratch)?;
-                        self.wtr.write_record(row1.iter().chain(&scratch))?;
-                    }
+                    validx.idx.seek(rowi as u64)?;
+                    validx.idx.read_byte_record(&mut scratch)?;
+                    self.wtr.write_record(row1.iter().chain(&scratch))?;
                 }
+            } else {
+                self.wtr.write_record(row1.iter().chain(&pad2))?;
             }
         }
 
@@ -331,10 +317,12 @@ impl Args {
         let rconf1 = Config::new(&Some(self.arg_input1.clone()))
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
+            .checkutf8(false)
             .select(self.arg_columns1.clone());
         let rconf2 = Config::new(&Some(self.arg_input2.clone()))
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
+            .checkutf8(false)
             .select(self.arg_columns2.clone());
 
         let mut rdr1 = rconf1.reader_file_stdin()?;
@@ -347,11 +335,12 @@ impl Args {
             rdr2,
             sel2,
             no_headers: rconf1.no_headers,
-            casei: self.flag_no_case,
+            casei: self.flag_ignore_case,
             nulls: self.flag_nulls,
         })
     }
 
+    #[allow(clippy::unused_self)]
     fn get_selections<R: io::Read>(
         &self,
         rconf1: &Config,
@@ -364,12 +353,12 @@ impl Args {
         let select1 = rconf1.selection(headers1)?;
         let select2 = rconf2.selection(headers2)?;
         if select1.len() != select2.len() {
-            return fail!(format!(
-                "Column selections must have the same number of columns, \
-                 but found column selections with {} and {} columns.",
+            return fail_clierror!(
+                "Column selections must have the same number of columns, but found column \
+                 selections with {} and {} columns.",
                 select1.len(),
                 select2.len()
-            ));
+            );
         }
         Ok((select1, select2))
     }
@@ -377,8 +366,8 @@ impl Args {
 
 struct ValueIndex<R> {
     // This maps tuples of values to corresponding rows.
-    values: AHashMap<Vec<ByteString>, Vec<usize>>,
-    idx: Indexed<R, io::Cursor<Vec<u8>>>,
+    values:   AHashMap<Vec<ByteString>, Vec<usize>>,
+    idx:      Indexed<R, io::Cursor<Vec<u8>>>,
     num_rows: usize,
 }
 
@@ -455,7 +444,7 @@ impl<R> fmt::Debug for ValueIndex<R> {
                 .iter()
                 .map(|k| String::from_utf8(k.clone()).unwrap())
                 .collect::<Vec<_>>();
-            writeln!(f, "({}) => {:?}", keys.join(", "), rows)?;
+            writeln!(f, "({}) => {rows:?}", keys.join(", "))?;
         }
         Ok(())
     }
@@ -466,17 +455,22 @@ fn get_row_key(sel: &Selection, row: &csv::ByteRecord, casei: bool) -> Vec<ByteS
     sel.select(row).map(|v| transform(v, casei)).collect()
 }
 
+pub type ByteString = Vec<u8>;
+
 #[inline]
-fn transform(bs: &[u8], casei: bool) -> ByteString {
-    let s = unsafe { str::from_utf8_unchecked(bs) };
-    if casei {
-        let norm: String = s
-            .trim()
-            .chars()
-            .map(|c| c.to_lowercase().next().unwrap())
-            .collect();
-        norm.into_bytes()
+pub fn transform(bs: &[u8], casei: bool) -> ByteString {
+    if let Ok(s) = str::from_utf8(bs) {
+        if casei {
+            let norm: String = s
+                .trim()
+                .chars()
+                .map(|c| c.to_lowercase().next().unwrap())
+                .collect();
+            norm.into_bytes()
+        } else {
+            s.trim().as_bytes().to_vec()
+        }
     } else {
-        s.trim().as_bytes().to_vec()
+        bs.to_vec()
     }
 }
