@@ -2,18 +2,18 @@ static USAGE: &str = r#"
 Compute summary statistics & infers data types for each column in a CSV.
 
 Summary statistics includes sum, min/max/range, min/max length, mean, stddev, variance,
-nullcount, quartiles, interquartile range (IQR), lower/upper fences, skewness, median, 
+nullcount, sparsity, quartiles, interquartile range (IQR), lower/upper fences, skewness, median, 
 cardinality, mode/s & antimode/s, and median absolute deviation (MAD). Note that some
 statistics requires loading the entire file into memory, so they must be enabled explicitly. 
 
 By default, the following statistics are reported for *every* column in the CSV data:
-sum, min/max/range values, min/max length, mean, stddev, variance & nullcount. The default
-set of statistics corresponds to statistics that can be computed efficiently on a stream
-of data (i.e., constant memory) and can work with arbitrarily large CSV files.
+sum, min/max/range values, min/max length, mean, stddev, variance, nullcount & sparsity.
+The default set of statistics corresponds to statistics that can be computed efficiently
+on a stream of data (i.e., constant memory) and can work with arbitrarily large CSV files.
 
 The following additional statistics require loading the entire file into memory:
-cardinality, mode/antimode, median, quartiles and its related measures 
-(IQR, lower/upper fences & skewness) and MAD.
+cardinality, mode/antimode, median, MAD, quartiles and its related measures (IQR,
+lower/upper fences & skewness).
 
 "Antimode" is the least frequently occurring non-zero value and is the opposite of mode.
 It returns "*ALL" if all the values are unique, and only returns a preview of the first
@@ -56,7 +56,8 @@ stats options:
                               into 'qsv stats' will disable the use of indexing.
     --everything              Show all statistics available.
     --typesonly               Infer data types only and do not compute statistics.
-                              Automatically turns on --infer-dates for all columns.
+                              Note that if you want to infer dates, you'll still need to use
+                              the --infer-dates and --dates-whitelist options.
     --mode                    Show the mode/s & antimode/s. Multimodal-aware.
                               This requires loading all CSV data in memory.
     --cardinality             Show the cardinality.
@@ -169,12 +170,17 @@ pub struct Args {
 
 static INFER_DATE_FLAGS: once_cell::sync::OnceCell<Vec<bool>> = OnceCell::new();
 static DMY_PREFERENCE: AtomicBool = AtomicBool::new(false);
+static RECORD_COUNT: once_cell::sync::OnceCell<u64> = OnceCell::new();
+
+// number of milliseconds per day
+const MS_IN_DAY: f64 = 86_400_000.0;
+// number of decimal places when rounding days
+// 5 decimal places give us millisecond precision
+const DAY_DECIMAL_PLACES: u32 = 5;
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
     if args.flag_typesonly {
-        args.flag_infer_dates = true;
-        args.flag_dates_whitelist = String::from("all");
         args.flag_everything = false;
         args.flag_mode = false;
         args.flag_cardinality = false;
@@ -184,7 +190,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     let mut wtr = Config::new(&args.flag_output).writer()?;
-    let (headers, stats) = match args.rconfig().indexed()? {
+    let fconfig = args.rconfig();
+    let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&fconfig).unwrap());
+    log::info!("scanning {record_count} records...");
+    let (headers, stats) = match fconfig.indexed()? {
         None => args.sequential_stats(&args.flag_dates_whitelist),
         Some(idx) => {
             if let Some(num_jobs) = args.flag_jobs {
@@ -359,8 +368,8 @@ impl Args {
             return csv::StringRecord::from(vec!["field", "type"]);
         }
 
-        // with --everything, we have 29 columns at most
-        let mut fields = Vec::with_capacity(29);
+        // with --everything, we have 30 columns at most
+        let mut fields = Vec::with_capacity(30);
         fields.extend_from_slice(&[
             "field",
             "type",
@@ -374,6 +383,7 @@ impl Args {
             "stddev",
             "variance",
             "nullcount",
+            "sparsity",
         ]);
         let all = self.flag_everything;
         if self.flag_median && !self.flag_quartiles && !all {
@@ -673,8 +683,8 @@ impl Stats {
 
         let typ = self.typ;
         // prealloc memory for performance
-        // we have 29 columns at most with --everything
-        let mut pieces = Vec::with_capacity(29);
+        // we have 30 columns at most with --everything
+        let mut pieces = Vec::with_capacity(30);
         let empty = String::new;
 
         // type
@@ -738,11 +748,10 @@ impl Stats {
                 pieces.push(timestamp_ms_to_rfc3339(v.mean() as i64, typ));
                 // instead of returning stdev in seconds, let's return it in
                 // days as it easier to handle
-                // 86_400_000 = number of milliseconds in a day.
                 // Round to at least 5 decimal places
                 pieces.push(round_num(
-                    v.stddev() / 86_400_000.0_f64,
-                    u32::max(round_places, 5),
+                    v.stddev() / MS_IN_DAY,
+                    u32::max(round_places, DAY_DECIMAL_PLACES),
                 ));
                 // we don't know how to compute variance on timestamps
                 // it appears the current algorithm we use is not suited to the large timestamp
@@ -762,12 +771,24 @@ impl Stats {
         let mut buffer = itoa::Buffer::new();
         pieces.push(buffer.format(self.nullcount).to_owned());
 
+        // sparsity
+        // stats is also called by the `schema` and `tojsonl` commands to infer a schema,
+        // sparsity is not required by those cmds and we don't have necessarily
+        // have record_count at this point, so just set sparsity to nullcount (div by 1) so
+        // we don't panic.
+        #[allow(clippy::cast_precision_loss)]
+        let sparsity: f64 = self.nullcount as f64 / *RECORD_COUNT.get().unwrap_or(&1) as f64;
+        let mut buffer = ryu::Buffer::new();
+        pieces.push(buffer.format(sparsity).to_owned());
+
         // median
+        let mut existing_median = None;
         if let Some(v) = self.median.as_mut().and_then(|v| {
             if let TNull | TString = typ {
                 None
             } else {
-                v.median()
+                existing_median = v.median();
+                existing_median
             }
         }) {
             if typ == TDateTime || typ == TDate {
@@ -784,12 +805,15 @@ impl Stats {
             if let TNull | TString = typ {
                 None
             } else {
-                v.mad()
+                v.mad(existing_median)
             }
         }) {
             if typ == TDateTime || typ == TDate {
                 // like stddev, return MAD in days
-                pieces.push(round_num(v / 86_400_000.0_f64, u32::max(round_places, 5)));
+                pieces.push(round_num(
+                    v / MS_IN_DAY,
+                    u32::max(round_places, DAY_DECIMAL_PLACES),
+                ));
             } else {
                 pieces.push(round_num(v, round_places));
             }
@@ -854,8 +878,8 @@ impl Stats {
                     pieces.push(timestamp_ms_to_rfc3339(q3 as i64, typ));
                     // return iqr in days - there are 86,400,000 ms in a day
                     pieces.push(round_num(
-                        (q3 - q1) / 86_400_000.0_f64,
-                        u32::max(round_places, 5),
+                        (q3 - q1) / MS_IN_DAY,
+                        u32::max(round_places, DAY_DECIMAL_PLACES),
                     ));
 
                     pieces.push(timestamp_ms_to_rfc3339(uif as i64, typ));
@@ -1271,12 +1295,9 @@ impl TypedMinMax {
                     Some((
                         timestamp_ms_to_rfc3339(*min, typ),
                         timestamp_ms_to_rfc3339(*max, typ),
-                        // 86_400_000 = number of milliseconds per day
+                        // return in days, not timestamp in milliseconds
                         #[allow(clippy::cast_precision_loss)]
-                        round_num(
-                            (*max - *min) as f64 / 86_400_000.0_f64,
-                            u32::max(round_places, 5),
-                        ),
+                        round_num((*max - *min) as f64 / MS_IN_DAY, u32::max(round_places, 5)),
                     ))
                 } else {
                     None
