@@ -5,8 +5,8 @@ By scanning the CSV first, it "smartly" infers the appropriate JSON data type
 for each column.
 
 It will infer a column as boolean if it only has a domain of two values,
-and the values are one of the following combinations, case-insensitive:
- "true"/"false"; "true"/null; "yes"/"no"; "yes"/null; "y"/"n" and "y"/null.  
+and the first character of the values are one of the following case-insensitive
+combinations: t/f; t/null; 1/0; 1/null; y/n & y/null are treated as true/false.
 
 For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_tojsonl.rs.
 
@@ -26,16 +26,17 @@ Common options:
     -o, --output <file>    Write output to <file> instead of stdout.
 "#;
 
-use std::{env::temp_dir, fs::File, path::Path};
+use std::{env::temp_dir, fmt::Write, fs::File, path::Path, str::FromStr};
 
 use serde::Deserialize;
 use serde_json::{Map, Value};
+use strum_macros::EnumString;
 use uuid::Uuid;
 
 use super::schema::infer_schema_from_stats;
 use crate::{
     config::{Config, Delimiter},
-    util, CliResult,
+    util, CliError, CliResult,
 };
 
 #[derive(Deserialize, Clone)]
@@ -46,6 +47,22 @@ struct Args {
     flag_output:    Option<String>,
 }
 
+impl From<std::fmt::Error> for CliError {
+    fn from(err: std::fmt::Error) -> CliError {
+        CliError::Other(err.to_string())
+    }
+}
+
+#[derive(PartialEq, EnumString)]
+#[strum(ascii_case_insensitive)]
+enum JsonlType {
+    Boolean,
+    String,
+    Number,
+    Integer,
+    Null,
+}
+
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let preargs: Args = util::get_args(USAGE, argv)?;
     let mut args = preargs.clone();
@@ -104,6 +121,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else {
         conf.reader()?
     };
+
+    // TODO: instead of abusing csv writer to write jsonl file
+    // just use a normal buffered writer
     let mut wtr = Config::new(&args.flag_output)
         .flexible(true)
         .no_headers(true)
@@ -111,10 +131,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .writer()?;
 
     let headers = rdr.headers()?.clone();
-    let mut record = csv::StringRecord::new();
 
     // create a vec lookup about inferred field data types
-    let mut field_type_vec: Vec<String> = Vec::with_capacity(headers.len());
+    let mut field_type_vec: Vec<JsonlType> = Vec::with_capacity(headers.len());
     for (_field_name, field_def) in properties_map.iter() {
         let Some(field_map) = field_def.as_object() else { return fail!("Cannot create field map") };
         let prelim_type = field_map.get("type").unwrap();
@@ -123,73 +142,122 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // log::debug!("prelim_type: {prelim_type} field_values_enum: {field_values_enum:?}");
 
         // check if a field has a boolean data type
-        if let Some(values) = field_values_enum {
-            if let Some(vals) = values.as_array() {
+        // by checking its enum constraint
+        if let Some(domain) = field_values_enum {
+            if let Some(vals) = domain.as_array() {
+                // if this field only has a domain of two values
                 if vals.len() == 2 {
                     let val1 = if vals[0].is_null() {
-                        "null".to_string()
+                        '_'
                     } else {
-                        vals[0].as_str().unwrap().trim().to_lowercase()
+                        // check the first domain value, if its a string
+                        // get the first character of val1 lowercase
+                        if let Some(str_val) = vals[0].as_str() {
+                            first_lower_char(str_val)
+                        } else if let Some(int_val) = vals[0].as_u64() {
+                            // else, its an integer (as we only do enum constraints
+                            // for string and integers), and see if its 1 or 0
+                            match int_val {
+                                1 => '1',
+                                0 => '0',
+                                _ => '*', // its something else
+                            }
+                        } else {
+                            '*'
+                        }
                     };
+                    // same as above, but for the 2nd domain value
                     let val2 = if vals[1].is_null() {
-                        "null".to_string()
+                        '_'
+                    } else if let Some(str_val) = vals[1].as_str() {
+                        first_lower_char(str_val)
+                    } else if let Some(int_val) = vals[1].as_u64() {
+                        match int_val {
+                            1 => '1',
+                            0 => '0',
+                            _ => '*',
+                        }
                     } else {
-                        vals[1].as_str().unwrap().trim().to_lowercase()
+                        '*'
                     };
                     // log::debug!("val1: {val1} val2: {val2}");
-                    if let ("true", "false")
-                    | ("false", "true")
-                    | ("yes", "no")
-                    | ("no", "yes")
-                    | ("y", "n")
-                    | ("n", "y")
-                    | ("true", "null")
-                    | ("null", "true")
-                    | ("yes", "null")
-                    | ("null", "yes")
-                    | ("y", "null")
-                    | ("null", "y") = (val1.as_str(), val2.as_str())
+
+                    // check if the domain of two values is truthy or falsy
+                    // i.e. if first character, case-insensitive is "t", "1" or "y" - truthy
+                    // "f", "0", "n" or null - falsy
+                    // if it is, infer a boolean field
+                    if let ('t', 'f' | '_')
+                    | ('f' | '_', 't')
+                    | ('1', '0' | '_')
+                    | ('0' | '_', '1')
+                    | ('y', 'n' | '_')
+                    | ('n' | '_', 'y') = (val1, val2)
                     {
-                        field_type_vec.push("boolean".to_string());
+                        field_type_vec.push(JsonlType::Boolean);
                         continue;
                     }
                 }
             }
         }
-        // its not a boolean, so its a Number, String or Null
-        let temp_type = prelim_type.clone();
-        let temp_str = temp_type.as_array().unwrap()[0]
-            .as_str()
-            .unwrap_or_default();
-        if temp_str == "integer" {
-            field_type_vec.push("number".to_string());
-        } else {
-            field_type_vec.push(temp_str.to_string());
-        }
+
+        // ok to use index access and unwrap here as we know
+        // we have at least one element in the prelim_type as_array
+        field_type_vec.push(
+            JsonlType::from_str(
+                prelim_type.as_array().unwrap()[0]
+                    .as_str()
+                    .unwrap_or("null"),
+            )
+            .unwrap_or(JsonlType::String),
+        );
     }
 
     // amortize allocs
+    let mut record = csv::StringRecord::new();
     #[allow(unused_assignments)]
     let mut temp_str = String::with_capacity(100);
+    #[allow(unused_assignments)]
+    let mut temp_str2 = String::with_capacity(50);
 
-    // write jsonl file
+    // TODO: see if its worth it to do rayon here after benchmarking
+    // with large files. We have --jobs option, but we only pass it
+    // thru to stats/frequency to infer data types & enum constraints.
+    //
+    // now that we have type mappings, iterate thru input csv
+    // and write jsonl file
     while rdr.read_record(&mut record)? {
-        use std::fmt::Write as _;
-
         temp_str.clear();
-        let _ = write!(temp_str, "{{");
+        record.trim();
+        write!(temp_str, "{{")?;
         for (idx, field) in record.iter().enumerate() {
-            let field_val = match field_type_vec[idx].as_str() {
-                "string" => format!(r#""{}""#, field.escape_default()),
-                "number" => field.to_string(),
-                "boolean" => match field.trim().to_lowercase().as_str() {
-                    "true" | "yes" | "y" => "true".to_string(),
-                    _ => "false".to_string(),
-                },
-                "null" => "null".to_string(),
-                _ => "unknown".to_string(),
+            let field_val = if let Some(field_type) = field_type_vec.get(idx) {
+                match field_type {
+                    JsonlType::String => {
+                        if field.is_empty() {
+                            "null"
+                        } else {
+                            temp_str2 = format!(r#""{}""#, field.escape_default());
+                            &temp_str2
+                        }
+                    }
+                    JsonlType::Null => "null",
+                    JsonlType::Integer | JsonlType::Number => field,
+                    JsonlType::Boolean => {
+                        if let 't' | 'y' | '1' = first_lower_char(field) {
+                            "true"
+                        } else {
+                            "false"
+                        }
+                    }
+                }
+            } else {
+                "null"
             };
-            let _ = write!(temp_str, r#""{}":{field_val},"#, &headers[idx]);
+            if field_val.is_empty() {
+                write!(temp_str, r#""{}":null,"#, &headers[idx])?;
+            } else {
+                write!(temp_str, r#""{}":{field_val},"#, &headers[idx])?;
+            }
         }
         temp_str.pop(); // remove last comma
         temp_str.push('}');
@@ -200,3 +268,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     Ok(wtr.flush()?)
 }
+
+#[inline]
+fn first_lower_char(field_str: &str) -> char {
+    field_str
+        .trim_start()
+        .chars()
+        .next()
+        .unwrap_or('_')
+        .to_ascii_lowercase()
+}
