@@ -1,5 +1,4 @@
-use crate::workdir::Workdir;
-use crate::{qcheck, CsvData};
+use crate::{qcheck, workdir::Workdir, CsvData};
 
 /// This tests whether `qsv count` gets the right answer.
 ///
@@ -120,3 +119,22 @@ fn prop_count_noheaders_indexed_env() {
     }
     qcheck(p as fn(CsvData) -> bool);
 }
+
+#[test]
+fn count_width() {
+    let wrk = Workdir::new("count_width");
+    wrk.create_indexed(
+        "in.csv",
+        vec![
+            svec!["letter", "number"],
+            svec!["alpha", "13"],
+            svec!["beta", "24"],
+        ],
+    );
+    let mut cmd = wrk.command("count");
+    cmd.arg("--width").arg("in.csv");
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = "2;9";
+    assert_eq!(got, expected.to_string());
+}
