@@ -30,6 +30,36 @@ fn py_map() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn py_map_error() {
+    let wrk = Workdir::new("py");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["letter", "number"],
+            svec!["a", "13"],
+            svec!["b", "24"],
+            svec!["c", "72"],
+            svec!["d", "7"],
+        ],
+    );
+    let mut cmd = wrk.command("py");
+    cmd.arg("map")
+        .arg("inc")
+        .arg("integerthis(number) + 1")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "number", "inc"],
+        svec!["a", "13", "<ERROR>"],
+        svec!["b", "24", "<ERROR>"],
+        svec!["c", "72", "<ERROR>"],
+        svec!["d", "7", "<ERROR>"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn py_map_builtins() {
     let wrk = Workdir::new("py");
@@ -90,6 +120,36 @@ fn py_map_math() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn py_map_datetime() {
+    let wrk = Workdir::new("py");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["letter", "datecol"],
+            svec!["a", "2019-12-04"],
+            svec!["b", "2001-01-03"],
+            svec!["c", "1991-07-04"],
+            svec!["d", "2021-01-04"],
+        ],
+    );
+    let mut cmd = wrk.command("py");
+    cmd.arg("map")
+        .arg("fivedaysago")
+        .arg("datetime.date.fromisoformat(datecol) - datetime.timedelta(days=5)")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "datecol", "fivedaysago"],
+        svec!["a", "2019-12-04", "2019-11-29"],
+        svec!["b", "2001-01-03", "2000-12-29"],
+        svec!["c", "1991-07-04", "1991-06-29"],
+        svec!["d", "2021-01-04", "2020-12-30"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn py_map_userhelper() {
     let wrk = Workdir::new("py");
@@ -350,6 +410,39 @@ fn py_filter() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn py_filter_error() {
+    let wrk = Workdir::new("py");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["letter", "number"],
+            svec!["a", "13"],
+            svec!["b", "24"],
+            svec!["c", "72"],
+            svec!["d", "7"],
+        ],
+    );
+    let mut cmd = wrk.command("py");
+    cmd.arg("filter")
+        .arg("integerthis(number) > 14")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["letter", "number"],
+        svec!["a", "13"],
+        svec!["b", "24"],
+        svec!["c", "72"],
+        svec!["d", "7"],
+    ];
+    assert_eq!(got, expected);
+
+    wrk.assert_err(&mut cmd);
+    let stderr_string = wrk.output_stderr(&mut cmd);
+    assert!(stderr_string.ends_with("Python errors encountered: 4\n"));
+}
+
 #[test]
 fn py_format() {
     let wrk = Workdir::new("py");
@@ -378,6 +471,49 @@ fn py_format() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn py_format_with_conditionals() {
+    let wrk = Workdir::new("py");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["qty", "fruit", "unit cost"],
+            svec!["20.5", "mangoes", "5"],
+            svec!["10", "bananas", "20"],
+            svec!["3", "strawberries", "3.50"],
+        ],
+    );
+    let mut cmd = wrk.command("py");
+    cmd.arg("map")
+        .arg("formatted")
+        .arg(r#"f"""{qty} {fruit} cost ${(float(unit_cost) * float(qty)):.2f}. Its quite {"cheap" if ((float(unit_cost) * float(qty)) < 20.0) else "expensive"}!""""#)
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["qty", "fruit", "unit cost", "formatted"],
+        svec![
+            "20.5",
+            "mangoes",
+            "5",
+            "20.5 mangoes cost $102.50. Its quite expensive!"
+        ],
+        svec![
+            "10",
+            "bananas",
+            "20",
+            "10 bananas cost $200.00. Its quite expensive!"
+        ],
+        svec![
+            "3",
+            "strawberries",
+            "3.50",
+            "3 strawberries cost $10.50. Its quite cheap!"
+        ],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn py_format_header_with_invalid_chars() {
     let wrk = Workdir::new("py");
